unit FrmMidiSelect;

interface

uses

  WinApi.mmSystem,

  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, System.IniFiles,

  knobs2013, midi_defs;

type

  TFormMidiSelect = class(TForm)
    PanelMain: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    ComboBoxInput: TComboBox;
    ComboBoxOutput: TComboBox;
    BitBtnOk: TBitBtn;
    BitBtnCancel: TBitBtn;
    GroupBoxNetMidi: TGroupBox;
    Label42: TLabel;
    main_midich: TKnobsNoKnob;
    main_display_midich: TKnobsDisplay;
    RadioGroupRpnMode: TRadioGroup;
    procedure ComboBoxInputChange(Sender: TObject);
    procedure ComboBoxOutputChange(Sender: TObject);
    procedure RadioGroupRpnModeClick(Sender: TObject);
    procedure main_midichValueChanged(const aSender: TObject; const aPath, aControlType: string; aValue: Double;
      IsFinal, IsAutomation: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    FInitialized          : Boolean;
    FMidiInDeviceName     : string;
    FMidiOutDeviceName    : string;
    FMidiChannel          : Byte;
    FMidiRpnMode          : TRpnMode;
    FOldMidiInDeviceName  : string;
    FOldMidiOutDeviceName : string;
    FOldMidiChannel       : Byte;
    FOldMidiRpnMode       : TRpnMode;
  private
    procedure   SetMidiInDeviceName ( const aValue: string  );
    procedure   SetMidiOutDeviceName( const aValue: string  );
    procedure   SetMidiChannel      (       aValue: Byte    );
    procedure   SetMidiRpnMode      (       aValue: TRpnMode);
  private
    procedure   SaveSettings;
    procedure   RestoreSettings;
  public
    procedure   Initialize;
    procedure   SaveToIni  ( anIniFile: TCustomIniFile; const aSection: string);
    procedure   LoadFromIni( anIniFile: TCustomIniFile; const aSection: string);
    function    Execute: Boolean;
  public
    property    MidiInDeviceName  : string   read FMidiInDeviceName  write SetMidiInDeviceName ;
    property    MidiOutDeviceName : string   read FMidiOutDeviceName write SetMidiOutDeviceName;
    property    MidiChannel       : Byte     read FMidiChannel       write SetMidiChannel;
    property    MidiRpnMode       : TRpnMode read FMidiRpnMode       write SetMidiRpnMode;
  end;


var

  FormMidiSelect: TFormMidiSelect;


implementation


//  User area - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

//  private

    procedure   TFormMidiSelect.SetMidiInDeviceName( const aValue: string);
    var
      i : Integer;
    begin
      with ComboBoxInput
      do begin
        ItemIndex := -1;

        for i := 0 to Items.Count - 1
        do begin
          if   UpperCase( aValue) = UpperCase( Items[ i])
          then begin
            ItemIndex := i;
            FMidiInDeviceName := Items[ i];
            Break;
          end;
        end;
      end;
    end;


    procedure   TFormMidiSelect.SetMidiOutDeviceName( const aValue: string);
    var
      i : Integer;
    begin
      with ComboBoxOutput
      do begin
        ItemIndex := -1;

        for i := 0 to Items.Count - 1
        do begin
          if   UpperCase( aValue) = UpperCase( Items[ i])
          then begin
            ItemIndex := i;
            FMidiOutDeviceName := Items[ i];
            Break;
          end;
        end;
      end;
    end;


    procedure   TFormMidiSelect.SetMidiChannel( aValue: Byte);
    begin
      if   aValue <> FMidiChannel
      then begin
        FMidiChannel             := aValue;
        main_midich.KnobPosition := aValue;
      end;
    end;


    procedure   TFormMidiSelect.SetMidiRpnMode( aValue: TRpnMode);
    begin
      if   aValue <> FMidiRpnMode
      then begin
        FMidiRpnMode := TRpnMode( RadioGroupRpnMode.ItemIndex);
        RadioGroupRpnMode.ItemIndex := Ord( aValue);
      end;
    end;


//  private

    procedure   TFormMidiSelect.SaveSettings;
    begin
      FOldMidiInDeviceName  := MidiInDeviceName ;
      FOldMidiOutDeviceName := MidiOutDeviceName;
      FOldMidiChannel       := MidiChannel      ;
      FOldMidiRpnMode       := MidiRpnMode      ;
    end;


    procedure   TFormMidiSelect.RestoreSettings;
    begin
    end;


//  public

    procedure   TFormMidiSelect.Initialize;
    Var
      i       : UINT;
      aInCaps : TMidiInCaps;
      aOutCaps: TMidiOutCaps;
    begin
      if not FInitialized
      then begin
        with ComboBoxInput
        do begin
          Clear;
          FMidiInDeviceName := '';

          if   MidiInGetNumDevs > 0
          then begin
            for i := 0 to MidiInGetNumDevs - 1
            do begin
              MidiInGetDevCaps( i, @ aInCaps, SizeOf( aInCaps));
              Items.Add( StrPas( aInCaps.szPName));
            end;

            if   Items.Count > 0
            then MidiInDeviceName := Items[ 0];
          end;
        end;

        with ComboBoxOutput
        do begin
          Clear;
          FMidiOutDeviceName := '';
          MidiOutGetDevCaps( MIDI_MAPPER, @ aOutCaps, SizeOf( aOutCaps));
          Items.Add( StrPas( aOutCaps.szPName));

          if   MidiOutGetNumDevs > 0
          then begin
            for i := 0 to MidiOutGetNumDevs - 1
            do begin
              MidiOutGetDevCaps( i, @ aOutCaps, SizeOf( aOutCaps));
              Items.Add( StrPas( aOutCaps.szPName));
            end;

            if   Items.Count > 0
            then MidiOutDeviceName := Items[ 0];
          end;
        end;

        FInitialized := True;
      end;

      MidiChannel := MIDI_CH_OMNI;
      MidiRpnMode := mrm14;
    end;


    procedure   TFormMidiSelect.SaveToIni( anIniFile: TCustomIniFile; const aSection: string);
    begin
      if   Assigned( anIniFile)
      and  ( aSection <> '')
      then begin
        anIniFile.EraseSection( aSection);
        anIniFile.WriteInteger( aSection, 'Left'              ,          Left             );
        anIniFile.WriteInteger( aSection, 'Top'               ,          Top              );
        anIniFile.WriteInteger( aSection, 'Width'             ,          Width            );
        anIniFile.WriteInteger( aSection, 'Height'            ,          Height           );
        anIniFile.WriteString ( aSection, 'MidiInDevicceName' ,          MidiInDeviceName );
        anIniFile.WriteString ( aSection, 'MidiOutDevicceName',          MidiOutDeviceName);
        anIniFile.WriteInteger( aSection, 'MidiChannel'       ,          MidiChannel      );
        anIniFile.WriteInteger( aSection, 'MidiRpnMode'       , Integer( MidiRpnMode)     );
      end;
    end;


    procedure   TFormMidiSelect.LoadFromIni( anIniFile: TCustomIniFile; const aSection: string);
    var
      L, T, W, H : Integer;
    begin
      if   Assigned( anIniFile)
      and  ( aSection <> '')
      then begin
        MidiInDeviceName  :=           anIniFile.ReadString ( aSection, 'MidiInDevicceName' ,          MidiInDeviceName );
        MidiOutDeviceName :=           anIniFile.ReadString ( aSection, 'MidiOutDevicceName',          MidiOutDeviceName);
        MidiChannel       :=           anIniFile.ReadInteger( aSection, 'MidiChannel'       ,          MidiChannel      );
        MidiRpnMode       := TRpnMode( anIniFile.ReadInteger( aSection, 'MidiRpnMode'       , Integer( MidiRpnMode))    );
        L                 :=           anIniFile.ReadInteger( aSection, 'Left'              ,          Left             );
        T                 :=           anIniFile.ReadInteger( aSection, 'Top'               ,          Top              );
        W                 :=           anIniFile.ReadInteger( aSection, 'Width'             ,          Width            );
        H                 :=           anIniFile.ReadInteger( aSection, 'Height'            ,          Height           );

        if ( L < 0) or ( L > Screen.Width  - 10) then L := 0;
        if ( T < 0) or ( T > Screen.Height - 10) then T := 0;
        if W > Screen.Width                      then W := Screen.Width ;
        if H > Screen.Height                     then H := Screen.Height;

        SetBounds( L, T, W, H);
      end;
    end;


    function    TFormMidiSelect.Execute: Boolean;
    begin
      if   not FInitialized
      then Result := False
      else begin
        SaveSettings;
        Result := ShowModal = mrOk;

        if   not Result
        then RestoreSettings;
      end;
    end;



// Delphi area  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

{$R *.dfm}

procedure TFormMidiSelect.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if   ModalResult <> mrOk
  then RestoreSettings;
end;

procedure TFormMidiSelect.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if   Key = VK_ESCAPE
  then begin
    ModalResult := mrCancel;
    Key         := 0;
  end
  else if Key = VK_RETURN
  then begin
    ModalResult := mrOk;
    Key         := 0;
  end
end;

procedure TFormMidiSelect.main_midichValueChanged(const aSender: TObject; const aPath, aControlType: string;
  aValue: Double; IsFinal, IsAutomation: Boolean);
begin
  MidiChannel := Round( aValue);
end;

procedure TFormMidiSelect.RadioGroupRpnModeClick(Sender: TObject);
begin
  MidiRpnMode := TRpnMode( RadioGroupRpnMode.ItemIndex);
end;

procedure TFormMidiSelect.ComboBoxInputChange(Sender: TObject);
begin
  with ComboBoxInput
  do begin
    if   ( ItemIndex >= 0         )
    and  ( ItemIndex < Items.count)
    then MidiInDeviceName := Items[ ItemIndex]
    else MidiInDeviceName := '';
  end;
end;

procedure TFormMidiSelect.ComboBoxOutputChange(Sender: TObject);
begin
  with ComboBoxOutput
  do begin
    if   ( ItemIndex >= 0         )
    and  ( ItemIndex < Items.count)
    then MidiOutDeviceName := Items[ ItemIndex]
    else MidiOutDeviceName := '';
  end;
end;

end.
